<?php

namespace Drupal\select2boxes\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsSelectWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\select2boxes\EntityCreationTrait;
use Drupal\select2boxes\MinSearchLengthTrait;

/**
 * Class SingleSelect2BoxesAutocompleteWidget.
 *
 * @FieldWidget(
 *   id = "select2boxes_autocomplete_single",
 *   label = @Translation("Select2 boxes (Single value)"),
 *   description = @Translation("Single select2 boxes for entity reference fields"),
 *   field_types = {
 *     "entity_reference",
 *   },
 *   multiple_values = TRUE
 * )
 * @package Drupal\select2boxes\Plugin\Field\FieldWidget
 */
class SingleSelect2BoxesAutocompleteWidget extends OptionsSelectWidget {
  use EntityCreationTrait;
  use MinSearchLengthTrait;

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $element['#attributes'] = [
      // Disable core autocomplete.
      'data-jquery-once-autocomplete'         => 'true',
      'data-select2-autocomplete-list-widget' => 'true',
      'class'                                 => ['select2-widget'],
    ];
    // Pass an additional data attribute
    // to let select2 JS know whether it should handle input
    // for auto-create or not.
    if ($this->getFieldSetting('handler_settings')['auto_create'] == TRUE) {
      $element['#attributes']['data-auto-create-entity'] = 'enabled';
    }
    // Handle entity "autocreate" option.
    if (!empty($form_state->getUserInput())) {
      $value = $form_state->getUserInput()[$this->fieldDefinition->getName()];
      if (!empty($value)) {
        $settings = $this->getFieldSettings();
        if ($settings['handler_settings']['auto_create']) {
          $target_type   = $settings['target_type'];
          $target_bundle = reset($settings['handler_settings']['target_bundles']);
          if (!isset($element['#options'][$value])) {
            // Prepare the data required for an entity creation.
            $definition = \Drupal::entityTypeManager()->getDefinition($target_type);
            $label_key  = $definition->getKey('label');
            $bundle_key = $definition->getKey('bundle');
            $label      = $value;
            $values     = [$label_key => $label, $bundle_key => $target_bundle];
            // Create entity.
            $value = $this->createEntity($target_type, $values);

            // Update element's form structure due to newly created option.
            $element['#options'] += [$value => $label];
            $element['#value'] = $value;
          }
        }
      }
    }
    $element['#multiple'] = $element['#needs_validation'] = FALSE;
    // Set the additional attribute for limiting
    // the search input visibility if specified.
    $this->limitSearchByMinLength($element['#attributes']);
    // Attach library.
    $element['#attached']['library'][] = 'select2boxes/widget';
    return $element;
  }

}
