<?php

namespace Drupal\select2boxes;

/**
 * Trait PreloadBuildTrait.
 *
 * @package Drupal\select2boxes
 */
trait PreloadBuildTrait {

  /**
   * Build preloaded entries list.
   *
   * @param string $count
   *   Number of entries will be preloaded, or empty string to load all.
   * @param array $settings
   *   Field settings array.
   * @param string $target_type
   *   Entity type ID.
   *
   * @return array
   *   Preloaded entries list array.
   */
  protected function buildPreLoaded($count, $settings, $target_type) {
    // Return empty array if the count is less than 1.
    $entities = [];
    if ($count <= 0 && $count != '') {
      return $entities;
    }
    // Prepare required keys, from the entity type definitions.
    $definition = \Drupal::entityTypeManager()->getDefinition($target_type);
    $bundle_key = $definition->getKey('bundle');
    $id_key     = $definition->getKey('id');
    $label_key  = $definition->getKey('label');
    $data_table = $definition->getDataTable();
    // Select data using the appropriate entity keys.
    $select = \Drupal::database()->select($data_table, 'preload');
    $select->fields('preload', [$id_key, $label_key]);
    // Add an optional condition by specific entity bundle(s).
    if (isset($settings['target_bundles']) && !empty($settings['target_bundles'])) {
      $bundles = array_values($settings['target_bundles']);
      $select->condition($bundle_key, $bundles, 'IN');
    }
    $entities = !$count
      ? $entities = $select->execute()->fetchAllKeyed()
      : $select->range(0, $count)->execute()->fetchAllKeyed();
    // Sort preloaded entities.
    asort($entities);
    return $entities;
  }

}
