<?php

namespace Drupal\Tests\select2boxes\FunctionalJavascript;

/**
 * Class Select2BoxesTests.
 *
 * @package Drupal\Tests\select2boxes\FunctionalJavascript
 * @group Select2Boxes
 */
class Select2BoxesTests extends Select2BoxesTestsBase {

  /**
   * Tests whether the widgets plugins are exist and accessible.
   */
  public function testWidgetsPlugins() {
    // Check if the plugin manager service exists.
    $this->assertTrue(\Drupal::hasService('plugin.manager.field.widget'));
    /** @var \Drupal\Core\Field\WidgetPluginManager $plugin_manager */
    $plugin_manager = \Drupal::service('plugin.manager.field.widget');
    // Check each plugin if it has definition under a plugin manager service.
    foreach (static::$pluginIds as $widget) {
      $this->assertTrue($plugin_manager->hasDefinition($widget));
    }
  }

  /**
   * Check whether all widgets are exist in the field widgets settings.
   */
  public function testWidgetsAdminExistence() {
    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);

    // Check for entity reference field widgets.
    $select = $this->getFieldById('edit-fields-field-tags-type');
    $options = $this->getFieldOptions($select);
    $this->assertNotEmpty($options);
    $this->assertContains(static::$pluginIds[1], $options);
    $this->assertContains(static::$pluginIds[2], $options);

    // Check if the newly created field is exist in the list.
    $select = $this->getFieldById('edit-fields-field-test-list-type');
    $this->assertNotNull($select);
    // Check for list field widgets.
    $options = $this->getFieldOptions($select);
    $this->assertNotEmpty($options);
    $this->assertContains(static::$pluginIds[0], $options);


    // Set the Select2Boxes widgets for both fields and submit the form.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_test_list][type]"]')
      ->setValue(static::$pluginIds[0]);
    $this->minkSession->wait(2000);
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_tags][type]"]')
      ->setValue(static::$pluginIds[1]);
    $this->minkSession->wait(2000);
    $this->saveForm();
    // Check if the submission is finishing correctly.
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Test single widget for entity reference fields.
   */
  public function testSingleWidget() {
    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);
    // Set single widget for the Tags field.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_tags][type]"]')
      ->setValue(static::$pluginIds[1]);
    $this->minkSession->wait(2000);
    $this->saveForm();
    $this->assertSession()->statusCodeEquals(200);
    // Now let's check the widgets style on a node creation page.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);
    $select = $this->getFieldById('edit-field-tags');
    $this->assertNotNull($select);
    // Check if all required html attributes are exist
    // for the entity reference field.
    $this->assertNotNull($select->getAttribute('data-jquery-once-autocomplete'));
    $this->assertNotNull($select->getAttribute('data-select2-autocomplete-list-widget'));
    $this->assertTrue($select->hasClass('select2-widget'));
  }

  /**
   * Test multiple widget for entity reference fields.
   */
  public function testMultipleWidget() {
    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);
    // Set multiple widget for the Tags field.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_tags][type]"]')
      ->setValue(static::$pluginIds[2]);
    $this->minkSession->wait(2000);
    $this->saveForm();
    $this->assertSession()->statusCodeEquals(200);
    // Now let's check the node creation page.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);
    $select = $this->getFieldById('edit-field-tags');
    $this->assertNotNull($select);
    // Check if all required html attributes are exist
    // for the entity reference field.
    $this->assertNotNull($select->getAttribute('data-jquery-once-autocomplete'));
    $this->assertNotNull($select->getAttribute('data-select2-multiple'));
    $this->assertNotNull($select->getAttribute('data-autocomplete-path'));
    $this->assertNotNull($select->getAttribute('data-field-name'));
    $this->assertTrue($select->hasClass('select2-widget'));
    $this->assertTrue($select->hasClass('select2-boxes-widget'));
  }

  /**
   * Test widget for a list type of fields.
   */
  public function testListWidget() {
    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);
    // Set list widget for the Test list field.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_test_list][type]"]')
      ->setValue(static::$pluginIds[0]);
    $this->minkSession->wait(2000);
    $this->saveForm();
    $this->assertSession()->statusCodeEquals(200);
    // Now let's check the node creation page.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);
    $select = $this->getFieldById('edit-field-test-list');
    $this->assertNotNull($select);
    // Check if all required html attributes are exist
    // for the list field.
    $this->assertNotNull($select->getAttribute('data-jquery-once-autocomplete'));
    $this->assertNotNull($select->getAttribute('data-select2-autocomplete-list-widget'));
    $this->assertTrue($select->hasClass('select2-widget'));
  }

  /**
   * Test preloading functionality for the multiple entity reference widgets.
   */
  public function testPreloading() {
    // Firstly generate a fake content.
    $this->generateDummyTerms('tags', 10);
    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);
    // Set multiple widget for the Tags field.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_tags][type]"]')
      ->setValue(static::$pluginIds[2]);
    $this->minkSession->wait(2000);
    // Check for the summary text about preloading option.
    $this->assertTextHelper('Preloading disabled', FALSE);
    // Open the settings form for the Tags field.
    $this->click('input[name="field_tags_settings_edit"]');
    $this->minkSession->wait(2000);
    // Enable preloading via checking the checkbox field.
    $this->minkSession
      ->getPage()
      ->checkField('fields[field_tags][settings_edit_form][third_party_settings][select2boxes][enable_preload]');
    $this->minkSession->wait(500);
    // Set 5 rows to be preloaded.
    $this->minkSession
      ->getPage()
      ->fillField('fields[field_tags][settings_edit_form][third_party_settings][select2boxes][preload_count]', '5');
    // Submit the settings form.
    $this->click('input[name="field_tags_plugin_settings_update"]');
    $this->minkSession->wait(2000);
    // Check for summary text updates,
    // according to the specified number of preload entries.
    $this->assertTextHelper('Number of preloaded entries: 5', FALSE);
    // Submit the entity form display settings.
    $this->saveForm();
    $this->minkSession->wait(2000);
    $this->assertSession()->statusCodeEquals(200);
    // Go to the node's creation form.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);

    // Trigger the opening dropdown via click on the search input field.
    $this->click('input[class="select2-search__field"]');
    $this->minkSession->wait(1000);
    // Find the list of results element on the page.
    $select = $this->getFieldById('select2-edit-field-tags-results');
    $this->assertNotNull($select);
    // Check if the number of results is equals to the 5 rows
    // (as was specified in the widget settings).
    $this->assertEquals(5, count($select->findAll('xpath', '//li')));
  }

  /**
   * Test limited search option functionality.
   */
  public function testLimitedSearch() {
    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);

    // Set list widget for the Test list field.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_test_list][type]"]')
      ->setValue(static::$pluginIds[0]);
    $this->minkSession->wait(2000);
    $this->saveForm();

    // Enable limited search option.
    $this->drupalGet('admin/config/user-interface/select2boxes');
    $this->assertSession()->statusCodeEquals(200);
    // Specify minimum length search to a BIGGER value
    // than we have in our "Allowed values" settings for the field.
    $edit = [
      'limited_search'        => '1',
      'minimum_search_length' => '4',
    ];
    $this->submitForm($edit, t('Save configuration'));
    $this->minkSession->wait(2000);
    $this->assertSession()->statusCodeEquals(200);
    // Go to the node's creation form.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);

    // Trigger opening dropdown.
    $this->minkSession
      ->getDriver()
      ->executeScript("jQuery('select[name=\"field_test_list\"]').select2('open');");
    // Check for NON-existing search input field.
    $search_input = $this->minkSession
      ->getPage()
      ->find('xpath', '//span[contains(@class, \'select2-search--dropdown\')]');
    $this->assertNotNull($search_input);
    $this->assertTrue($search_input->hasClass('select2-search--hide'));

    $this->drupalGet('admin/config/user-interface/select2boxes');
    $this->assertSession()->statusCodeEquals(200);
    // Specify minimum length search to the SAME value
    // than we have in our "Allowed values" settings for the field.
    $edit = [
      'limited_search'        => '1',
      'minimum_search_length' => '3',
    ];
    $this->submitForm($edit, t('Save configuration'));
    $this->minkSession->wait(2000);
    $this->assertSession()->statusCodeEquals(200);

    // Go to the node's creation form.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);

    // Trigger opening dropdown.
    $this->minkSession
      ->getDriver()
      ->executeScript("jQuery('select[name=\"field_test_list\"]').select2('open');");
    // Check for EXISTING search input field.
    $search_input = $this->minkSession
      ->getPage()
      ->find('xpath', '//span[contains(@class, \'select2-search--dropdown\')]');
    $this->assertNotNull($search_input);
    $this->assertFalse($search_input->hasClass('select2-search--hide'));
  }

  /**
   * Test entity auto-creation with limited search enabled.
   */
  public function testEntityAutoCreationWithLimitedSearch() {
    $this->drupalPostForm(
      'admin/config/user-interface/select2boxes',
      [
        'limited_search'        => '1',
        'minimum_search_length' => '4',
      ],
      t('Save')
    );

    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);

    // Set single widget for the Tags entity reference field.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_tags][type]"]')
      ->setValue(static::$pluginIds[1]);
    $this->minkSession->wait(2000);
    $this->saveForm();


    // Go to the node's creation form.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);

    // Trigger opening dropdown.
    $this->minkSession
      ->getDriver()
      ->executeScript("jQuery('select[name=\"field_tags\"]').select2('open');");
    // Check for EXISTING search input field.
    $search_input = $this->minkSession
      ->getPage()
      ->find('xpath', '//span[contains(@class, \'select2-search--dropdown\')]');
    $this->assertNotNull($search_input);
    $this->assertFalse($search_input->hasClass('select2-search--hide'));
  }

  /**
   * Test unlimited preloading behavior.
   */
  public function testUnlimitedPreloading() {
    // Firstly generate a fake content.
    $this->generateDummyTerms('tags', 10);
    // Go the the "Manage Form Display" form.
    $this->drupalGet(static::$manageFormDisplayPath);
    $this->assertSession()->statusCodeEquals(200);
    // Set multiple widget for the Tags field.
    $this->minkSession
      ->getPage()
      ->find('xpath', '//select[@name="fields[field_tags][type]"]')
      ->setValue(static::$pluginIds[2]);
    $this->minkSession->wait(2000);
    // Check for the summary text about preloading option.
    $this->assertTextHelper('Preloading disabled', FALSE);
    // Open the settings form for the Tags field.
    $this->click('input[name="field_tags_settings_edit"]');
    $this->minkSession->wait(2000);
    // Enable preloading via checking the checkbox field.
    $this->minkSession
      ->getPage()
      ->checkField('fields[field_tags][settings_edit_form][third_party_settings][select2boxes][enable_preload]');
    $this->minkSession->wait(500);
    // Do not specify the preload count value!
    // Submit the settings form.
    $this->click('input[name="field_tags_plugin_settings_update"]');
    $this->minkSession->wait(2000);
    // Check for summary text updates,
    // according to the NON-specified number of preload entries.
    $this->assertTextHelper('Number of preloaded entries: all', FALSE);
    // Submit the entity form display settings.
    $this->saveForm();
    $this->minkSession->wait(2000);
    $this->assertSession()->statusCodeEquals(200);
    // Go to the node's creation form.
    $this->drupalGet(static::$nodeAddPath);
    $this->assertSession()->statusCodeEquals(200);

    // Trigger the opening dropdown via click on the search input field.
    $this->click('input[class="select2-search__field"]');
    $this->minkSession->wait(1000);
    // Find the list of results element on the page.
    $select = $this->getFieldById('select2-edit-field-tags-results');
    $this->assertNotNull($select);
    // Check if the number of results is equals
    // to the 10 rows (as was generated).
    $this->assertEquals(10, count($select->findAll('xpath', '//li')));
  }

}
